import javax.microedition.lcdui.*;
import java.util.*;

public class MonitorCanvas extends Canvas implements CommandListener { 
    Monitor monitor;
    Monitor.State state;

    Font font;
    Font boldFont;
    int  fontHeight;
    int  fontWidth;
    int  screenWidth;
    int  screenHeight;
    int  arcX;
    int  arcY;
    int  arcDiameter;
    long total;
    int  scroll;
    boolean showDiagram;
    Timer     timer;
    TimerTask task;

    static final int PERIOD = 1000; // one second

    static final int[] colors = new int[] {
        0xff0000,
        0x00ff00,
        0x0000ff,
        0xff8080,
        0x80ffff,
        0xff00ff,
        0xdddd00,
        0xc0c0c0,
        0x8080ff,
        0xff8000
    };

        
    static final int WHITE = 0xffffff;
    static final int BLACK = 0x000000;

    class MonitorTask extends TimerTask 
    { 
        public void run() { 
            if (state.startTime != 0) { 
                repaint();
            }
        }
    }

    MonitorCanvas(Monitor monitor) 
    { 
        this.monitor = monitor;
        state = monitor.state;
        font = Font.getFont(Font.FACE_SYSTEM, Font.STYLE_PLAIN, Font.SIZE_SMALL);
        boldFont = Font.getFont(Font.FACE_SYSTEM, Font.STYLE_BOLD|Font.STYLE_UNDERLINED, Font.SIZE_SMALL);
	screenWidth = getWidth();
	screenHeight = getHeight(); 
        fontHeight = font.getHeight();
        fontWidth = boldFont.stringWidth("9.");        
        arcDiameter = screenWidth - fontWidth < screenHeight 
            ? screenWidth - fontWidth : screenHeight;
        showDiagram = true;
        arcX = (screenWidth - arcDiameter)/2;
        arcY = (screenHeight - arcDiameter)/2;
        for (int i = 0; i < state.nSteps; i++) { 
            total += state.elapsedTime[i];
        }
        setCommandListener(this);
        if (state.currentStep >= 0) { 
            addCommand(Monitor.STOP_CMD);
        } else { 
            if (state.nSteps != 0) { 
                addCommand(Monitor.RESET_CMD);
            }
            addCommand(Monitor.QUIT_CMD);
            addCommand(Monitor.HELP_CMD);
        }
        addCommand(Monitor.NEW_CMD);
        timer = new Timer();
        task = new MonitorTask();
        timer.scheduleAtFixedRate(task, PERIOD, PERIOD);
        Display.getDisplay(monitor).setCurrent(this);
    }

    void appendDigits(StringBuffer buf, int val, int width) 
    {
        String s = Integer.toString(val);
        int len = s.length();
        while (len < width) { 
            buf.append('0');
            len += 1;
        }
        buf.append(s);
    }
    
    String getTime(long elapsed) 
    {
        int hours = (int)(elapsed / (60*60*1000));
        int minutes = (int)(elapsed / (60*1000) % 60);
        int seconds = (int)(elapsed / 1000 % 60);
        StringBuffer buf = new StringBuffer();
        appendDigits(buf, hours, 2);
        buf.append(':');
        appendDigits(buf, minutes, 2);
        buf.append(':');
        appendDigits(buf, seconds, 2);
        if (state.currentStep < 0) { 
            buf.append('.');
            appendDigits(buf, (int)(elapsed%1000/10), 2);
        }
        return buf.toString();
    }

    protected void paint(Graphics g) 
    {        
        g.setColor(WHITE);
        g.fillRect(0, 0, screenWidth, screenHeight);        
        long running = 0;
        if (state.currentStep >= 0) { 
            running = System.currentTimeMillis() - state.startTime;
        }
        g.setColor(BLACK);
        if (showDiagram) { 
            g.drawArc(arcX, arcY, arcDiameter, arcDiameter, 0, 360);        
            int angle = 0;
            for (int i = 0; i < state.nSteps; i++) { 
                long elapsed = state.elapsedTime[i];
                if (i == state.currentStep) { 
                    elapsed += running;
                }
                if (elapsed != 0) { 
                    int part = (int)(elapsed*360/(total + running));
                    g.setColor(colors[i]);
                    g.fillArc(arcX, arcY, arcDiameter, arcDiameter, angle, part);
                    angle += part;
                }
            }
            if (angle > 0 && angle < 360 && state.nSteps > 0) { 
                g.fillArc(arcX, arcY, arcDiameter, arcDiameter, angle, 360 - angle);
            }            
        }    
        for (int i = scroll; i < state.nSteps; i++) { 
            long elapsed = state.elapsedTime[i];
            if (i == state.currentStep) { 
                elapsed += running;
                g.setFont(boldFont);
            } else { 
                g.setFont(font);
            }
            g.setColor(colors[i]);
            g.drawString(Integer.toString(i) + ". ", 0, (i-scroll)*fontHeight,
                         Graphics.LEFT|Graphics.TOP);
            g.setColor(BLACK);
            g.drawString(getTime(elapsed),
                         fontWidth, (i-scroll)*fontHeight,
                         Graphics.LEFT|Graphics.TOP);
        }
    }

    public void keyPressed(int keyCode)
    {
        switch (keyCode) 
        {
        case KEY_STAR:
            newStep();
            break;
        case KEY_POUND:
            stop();
            break;
        case KEY_NUM0:
        case KEY_NUM1:
        case KEY_NUM2:
        case KEY_NUM3:
        case KEY_NUM4:
        case KEY_NUM5:
        case KEY_NUM6:
        case KEY_NUM7:
        case KEY_NUM8:
        case KEY_NUM9:
        {
            long now = System.currentTimeMillis();
            if (state.currentStep >= 0) { 
                long delta = now - state.startTime;
                state.elapsedTime[state.currentStep] += delta;
                total += delta;
            } else { 
                addCommand(Monitor.STOP_CMD);
                removeCommand(Monitor.RESET_CMD);
                removeCommand(Monitor.QUIT_CMD);
                removeCommand(Monitor.HELP_CMD);
            }
            state.currentStep = keyCode - KEY_NUM0;
            if (state.currentStep >= state.nSteps) { 
                state.nSteps = state.currentStep+1;
            }
            state.startTime = now;
            monitor.flushStore();
            repaint();
            break;
        }
        default:
            switch (getGameAction(keyCode)) { 
            case UP:
            case LEFT:
                if (scroll > 0) { 
                    scroll -= 1;
                } else { 
                    scroll = state.nSteps;
                }
                repaint();
                break;
            case RIGHT:
            case DOWN:
                if (scroll < state.nSteps) { 
                    scroll += 1;
                }
                repaint();
                break;
            case FIRE:
                showDiagram = !showDiagram;
                repaint();
                break;
            }
        }
    }

    void stop() 
    { 
        if (state.currentStep >= 0) {
            long now = System.currentTimeMillis();
            long delta = now - state.startTime;
            state.elapsedTime[state.currentStep] += delta;
            total += delta;
            addCommand(Monitor.RESET_CMD);
            addCommand(Monitor.HELP_CMD);
            addCommand(Monitor.QUIT_CMD);
            removeCommand(Monitor.STOP_CMD);
            state.currentStep = -1;
            monitor.flushStore();
            repaint();
        }
    }

    void reset() 
    {
        for (int i = 0; i < state.nSteps; i++) { 
            state.elapsedTime[i] = 0;
        }
        removeCommand(Monitor.RESET_CMD);
        removeCommand(Monitor.STOP_CMD);
        addCommand(Monitor.HELP_CMD);
        addCommand(Monitor.QUIT_CMD);
        total = 0;
        state.currentStep = -1;
        state.nSteps = 0;
        monitor.flushStore();
        repaint();    
    }

    void newStep() 
    {
        if (state.nSteps < Monitor.MAX_STEPS) { 
            long now = System.currentTimeMillis();
            if (state.currentStep >= 0) { 
                long delta = now - state.startTime;
                state.elapsedTime[state.currentStep] += delta;
                total += delta;
            } else { 
                removeCommand(Monitor.RESET_CMD);
                removeCommand(Monitor.QUIT_CMD);
                removeCommand(Monitor.HELP_CMD);
                addCommand(Monitor.STOP_CMD);
            }
            state.currentStep = state.nSteps++;
            state.startTime = now;
            monitor.flushStore();
            repaint();
            if (state.nSteps == Monitor.MAX_STEPS) { 
                removeCommand(Monitor.NEW_CMD);
            }
        } 
    }

    public void commandAction(Command c, Displayable d)  
    {
        if (c == Monitor.STOP_CMD) {
            stop();
        } else if (c == Monitor.RESET_CMD) { 
            reset();
        } else if (c == Monitor.NEW_CMD) {
            newStep();
        } else if (c == Monitor.HELP_CMD) { 
            new ContextHelp(monitor, this);
        } else if (c == Monitor.QUIT_CMD) { 
            task.cancel();
            monitor.quit();
        }
    }
}